import dash_ag_grid as dag
from dash import Dash, html, Input, Output
from . import utils
import json
from dash.testing.wait import until


def test_cu001_custom_components(dash_duo):
    app = Dash(
        __name__,
        meta_tags=[
            {
                "http-equiv": "content-security-policy",
                "content": "default-src 'self'; script-src 'self' 'unsafe-inline';"
                " style-src https://* 'self' 'unsafe-inline'; "
                "font-src data: https://* 'self' 'unsafe-inline';"
                "img-src data: https://* 'self'",
            }
        ],
    )

    data = [
        {
            "ticker": "AAPL",
            "company": "Apple",
            "price": 154.98500061035156,
            "volume": "Low",
            "binary": False,
            "buy": {"children": "buy", "className": "btn btn-success"},
            "sell": {"children": "sell", "className": "btn btn-danger"},
            "action": "buy",
            "test": {"className": "btn btn-warning"},
        }
    ]

    columnDefs = [
        {
            "headerName": "Stock Ticker",
            "field": "ticker",
            "cellRenderer": "stockLink",
            "tooltipField": "ticker",
        },
        {"headerName": "Company", "field": "company", "filter": True},
        {
            "headerName": "Last Close Price",
            "type": "rightAligned",
            "field": "price",
            "valueFormatter": {"function": """d3.format("($,.2f")(params.value)"""},
            "editable": True,
        },
        {
            "headerName": "Volume",
            "type": "rightAligned",
            "field": "volume",
            "cellRenderer": "tags",
            "editable": True,
        },
        {
            "field": "binary",
            "cellRenderer": "checkbox",
        },
        {"field": "buy", "cellRenderer": "myCustomButton"},
        {"field": "sell", "cellRenderer": "myCustomButton"},
        {
            "field": "action",
            "cellRenderer": "customDropdown",
            "cellEditorParams": {
                "values": ["buy", "sell", "hold"],
            },
        },
        {"field": "test", "cellRenderer": "myCustomButton2"},
    ]

    defaultColDef = {
        "filter": "agNumberColumnFilter",
        "resizable": True,
        "sortable": True,
        "editable": False,
        "tooltipComponent": "myCustomTooltip",
    }

    table = dag.AgGrid(
        id="portfolio-grid",
        className="ag-theme-alpine-dark",
        columnDefs=columnDefs,
        rowData=data,
        columnSize="sizeToFit",
        defaultColDef=defaultColDef,
    )

    app.layout = html.Div(
        [table, html.Div(id="cellValueChanged"), html.Div(id="cellRendererData")],
    )

    @app.callback(
        Output("cellValueChanged", "children"),
        Input("portfolio-grid", "cellValueChanged"),
        prevent_initial_call=True,
    )
    def showChange(n):
        return json.dumps(n)

    @app.callback(
        Output("cellRendererData", "children"),
        Input("portfolio-grid", "cellRendererData"),
        prevent_initial_call=True,
    )
    def showChange(d):
        if d:
            return json.dumps(d["value"])

    dash_duo.start_server(app)

    grid = utils.Grid(dash_duo, "portfolio-grid")

    grid.wait_for_cell_text(0, 1, "Apple")

    ### testing components
    assert (
        grid.get_cell_html(0, 0).strip()
        == """<a href="https://finance.yahoo.com/quote/AAPL" target="AAPL">AAPL</a>"""
    )
    assert (
        grid.get_cell_html(0, 3).strip()
        == """<div style="width: 100%; height: 100%; padding: 5px; display: flex; justify-content: center; align-items: center;"><div style="background-color: red; border-radius: 15px;">Low</div></div>"""
    )
    assert (
        grid.get_cell_html(0, 4).strip()
        == """<div style="width: 100%; height: 100%; padding: 5px; display: flex; justify-content: center; align-items: center;"><input type="checkbox" style="cursor: pointer;"></div>"""
    )
    assert (
        grid.get_cell_html(0, 5).strip()
        == """<div style="width: 100%; height: 100%; padding: 5px; display: flex; justify-content: center; align-items: center;"><button class="btn btn-success">buy</button></div>"""
    )
    assert (
        grid.get_cell_html(0, 6).strip()
        == """<div style="width: 100%; height: 100%; padding: 5px; display: flex; justify-content: center; align-items: center;"><button class="btn btn-danger">sell</button></div>"""
    )
    assert (
        grid.get_cell_html(0, 7).strip()
        == """<div style="width: 100%; height: 100%; padding: 5px; display: flex; justify-content: center; align-items: center;"><select><option>buy</option><option>sell</option><option>hold</option></select></div>"""
    )
    grid.element_click_cell_button(0, 6)

    until(
        lambda: '"rowIndex": 0, "rowId": "0", "data": {"ticker": '
        '"AAPL", "company": "Apple", "price": 154.98500061035'
        '156, "volume": "Low", "binary": false, "buy": {"child'
        'ren": "buy", "className": "btn btn-success"}, "sell":'
        ' {"children": "sell", "className": "btn btn-danger", '
        '"n_clicks": null}, "action": "sell", "test": '
        '{"className": "btn btn-warning"}}, "oldValue": "bu'
        'y", "value": "sell", "colId": "action"'
        in dash_duo.find_element("#cellValueChanged").get_attribute("innerText"),
        timeout=3,
    )
    grid.element_click_cell_button(0, 8)
    dash_duo.wait_for_text_to_equal("#cellRendererData", '{"data": "updated"}')



"""
Nested tables.
"""

import dash_ag_grid as dag
import dash
from dash import html, dcc
from . import utils
from dash.testing.wait import until
import requests


def test_ca001_custom_aggFunc_functions(dash_duo):
    app = dash.Dash(__name__)

    data = requests.get(
        r"https://www.ag-grid.com/example-assets/olympic-winners.json"
    ).json()

    columnDefs = [
        # Row group by country and by year is enabled.
        {
            "field": "country",
            "rowGroup": True,
            "hide": True,
            "suppressColumnsToolPanel": True,
        },
        {
            "field": "sport",
            "rowGroup": True,
            "hide": True,
            "suppressColumnsToolPanel": True,
        },
        {
            "field": "year",
            "pivot": True,
            "hide": True,
            "suppressColumnsToolPanel": True,
        },
        {"field": "gold", "sortable": True, "filter": True, "aggFunc": "sum"},
        {"field": "silver", "sortable": True, "filter": True, "aggFunc": "sum"},
        {
            "headerName": "ratio",
            "colId": "goldSilverRatio",
            "aggFunc": {"function": "ratioAggFunc(params)"},
            "valueGetter": {"function": "ratioValueGetter(params)"},
            "valueFormatter": {"function": "ratioFormatter(params)"},
        },
    ]

    app.layout = html.Div(
        [
            dcc.Markdown("Demonstration of row groupings in a Dash AG Grid."),
            dcc.Markdown("This grid groups first by country and then by year."),
            dag.AgGrid(
                columnDefs=columnDefs,
                rowData=data,
                defaultColDef=dict(
                    resizable=True,
                    rowSelection="multiple",
                    suppressAggFuncInHeader=True,
                ),
                id="grid",
                enableEnterpriseModules=True,
            ),
        ]
    )

    dash_duo.start_server(app)

    grid = utils.Grid(dash_duo, "grid")

    grid.wait_for_cell_text(0, 0, "United States\n(1109)")
    grid.wait_for_cell_text(0, 4, "552")
    grid.wait_for_cell_text(0, 5, "440")
    grid.wait_for_cell_text(0, 6, "1.25")

    grid.get_cell_expandable(0, 0).click()
    dash_duo.wait_for_text_to_equal(
        '#grid .ag-row-level-1 [aria-colindex="1"]', "Swimming\n(145)"
    )
    dash_duo.wait_for_text_to_equal('#grid .ag-row-level-1 [aria-colindex="5"]', "139")
    dash_duo.wait_for_text_to_equal('#grid .ag-row-level-1 [aria-colindex="6"]', "77")
    dash_duo.wait_for_text_to_equal('#grid .ag-row-level-1 [aria-colindex="7"]', "1.81")

def test_ca002_custom_aggFunc_functions(dash_duo):
    app = dash.Dash(__name__)

    data = requests.get(
        r"https://www.ag-grid.com/example-assets/olympic-winners.json"
    ).json()

    columnDefs = [
        # Row group by country and by year is enabled.
        {
            "field": "country",
            "rowGroup": True,
            "hide": True,
            "suppressColumnsToolPanel": True,
        },
        {
            "field": "sport",
            "rowGroup": True,
            "hide": True,
            "suppressColumnsToolPanel": True,
        },
        {
            "field": "year",
            "pivot": True,
            "hide": True,
            "suppressColumnsToolPanel": True,
        },
        {"field": "gold", "sortable": True, "filter": True, "aggFunc": "sum"},
        {"field": "silver", "sortable": True, "filter": True, "aggFunc": "sum"},
        {
            "headerName": "ratio",
            "colId": "goldSilverRatio",
            "aggFunc": "ratioAggFunc",
            "valueGetter": {"function": "ratioValueGetter(params)"},
            "valueFormatter": {"function": "ratioFormatter(params)"},
        },
    ]

    app.layout = html.Div(
        [
            dcc.Markdown("Demonstration of row groupings in a Dash AG Grid."),
            dcc.Markdown("This grid groups first by country and then by year."),
            dag.AgGrid(
                columnDefs=columnDefs,
                rowData=data,
                defaultColDef=dict(
                    resizable=True,
                    rowSelection="multiple",
                    suppressAggFuncInHeader=True,
                ),
                id="grid",
                enableEnterpriseModules=True,
                dashGridOptions={
                    "rowSelection": "multiple",
                    "suppressAggFuncInHeader": True,
                    "animateRows": False,
                    'aggFuncs': {'ratioAggFunc': {"function": "ratioAggFunc"}},
                }
            ),
        ]
    )

    dash_duo.start_server(app)

    grid = utils.Grid(dash_duo, "grid")

    grid.wait_for_cell_text(0, 0, "United States\n(1109)")
    grid.wait_for_cell_text(0, 4, "552")
    grid.wait_for_cell_text(0, 5, "440")
    grid.wait_for_cell_text(0, 6, "1.25")

    grid.get_cell_expandable(0, 0).click()
    dash_duo.wait_for_text_to_equal(
        '#grid .ag-row-level-1 [aria-colindex="1"]', "Swimming\n(145)"
    )
    dash_duo.wait_for_text_to_equal('#grid .ag-row-level-1 [aria-colindex="5"]', "139")
    dash_duo.wait_for_text_to_equal('#grid .ag-row-level-1 [aria-colindex="6"]', "77")
    dash_duo.wait_for_text_to_equal('#grid .ag-row-level-1 [aria-colindex="7"]', "1.81")


import dash_ag_grid as dag
from dash import Dash, html, dcc
from . import utils
import time
import dash_mantine_components

def test_cm001_context_menu(dash_duo):
    app = Dash(__name__)

    masterColumnDefs = [
        {
            "headerName": "Country",
            "field": "country",
        },
        {"headerName": "Region", "field": "region"},
        {"headerName": "Population", "field": "population"},
    ]
    rowData = [
        {
            "country": "China",
            "region": "Asia",
            "population": 1411778724,
        },
        {
            "country": "India",
            "region": "Asia",
            "population": 1383524897,
        },
        {
            "country": "United States",
            "region": "Americas",
            "population": 332593407,
        },
        {
            "country": "Indonesia",
            "region": "Asia",
            "population": 271350000,
        },
    ]

    app.layout = html.Div(
        [
            dag.AgGrid(
                id="grid",
                enableEnterpriseModules=True,
                columnDefs=masterColumnDefs,
                rowData=rowData,
                columnSize="sizeToFit",
                dashGridOptions={
                                 'getContextMenuItems': {'function': 'contextTest(params)'},
                                 'popupParent': {'function': 'setBody()'}
                                 },
            ),
        ]
    )

    dash_duo.start_server(app)

    grid = utils.Grid(dash_duo, "grid")

    grid.wait_for_cell_text(0, 0, "China")

    ### testing animations
    action = utils.ActionChains(dash_duo.driver)
    action.context_click(grid.get_cell(0, 0)).perform()
    dash_duo.find_element('body > .ag-popup')

    assert dash_duo.find_element('.ag-popup .ag-menu-option-part.ag-menu-option-text').text == 'Alert China'


"""
Nested tables.
"""

import dash_ag_grid as dag
import dash
from dash import Input, Output, html, dcc, Dash
from . import utils
from dash.testing.wait import until
import time


def test_rm001_row_master(dash_duo):
    app = Dash(__name__)

    masterColumnDefs = [
        {
            "headerName": "Country",
            "field": "country",
            "cellRenderer": "agGroupCellRenderer",
        },
        {"headerName": "Region", "field": "region"},
        {"headerName": "Population", "field": "population"},
    ]
    detailColumnDefs = [
        {"headerName": "City", "field": "city"},
        {"headerName": "Pop. (City proper)", "field": "population_city"},
        {"headerName": "Pop. (Metro area)", "field": "population_metro"},
    ]
    rowData = [
        {
            "country": "China",
            "region": "Asia",
            "population": 1411778724,
            "cities": [
                {"city": "Shanghai", "population_city": 24870895, "population_metro": "NA"},
                {"city": "Beijing", "population_city": 21893095, "population_metro": "NA"},
                {
                    "city": "Chongqing",
                    "population_city": 32054159,
                    "population_metro": "NA",
                },
            ],
        },
        {
            "country": "India",
            "region": "Asia",
            "population": 1383524897,
            "cities": [
                {
                    "city": "Delhi",
                    "population_city": 16753235,
                    "population_metro": 29000000,
                },
                {
                    "city": "Mumbai",
                    "population_city": 12478447,
                    "population_metro": 24400000,
                },
                {
                    "city": "Kolkata",
                    "population_city": 4496694,
                    "population_metro": 14035959,
                },
            ],
        },
        {
            "country": "United States",
            "region": "Americas",
            "population": 332593407,
            "cities": [
                {
                    "city": "New York",
                    "population_city": 8398748,
                    "population_metro": 19303808,
                },
                {
                    "city": "Los Angeles",
                    "population_city": 3990456,
                    "population_metro": 13291486,
                },
                {
                    "city": "Chicago",
                    "population_city": 2746388,
                    "population_metro": 9618502,
                },
            ],
        },
        {
            "country": "Indonesia",
            "region": "Asia",
            "population": 271350000,
            "cities": [
                {
                    "city": "Jakarta",
                    "population_city": 10154134,
                    "population_metro": 33430285,
                },
            ],
        },
    ]

    app.layout = html.Div(
        [
            dag.AgGrid(
                id="grid",
                enableEnterpriseModules=True,
                columnDefs=masterColumnDefs,
                rowData=rowData,
                columnSize="sizeToFit",
                masterDetail=True,
                detailCellRendererParams={
                    "detailGridOptions": {
                        "columnDefs": detailColumnDefs,
                    },
                    "detailColName": "cities",
                    "suppressCallback": True,
                },
                dashGridOptions={"detailRowAutoHeight": True,
                                 'isRowMaster': {'function': 'params ? params.country == "China" : false'}},
            ),
        ]
    )

    dash_duo.start_server(app)

    grid = utils.Grid(dash_duo, "grid")
    grid.wait_for_cell_text(0, 0, "China")
    grid.get_cell_expandable(0, 0)

    try:
        grid.get_cell_expandable(1, 0)
        ### expandable row was found on India
        assert False
    except:
        ### expandable row was not found on India
        assert True




import dash_ag_grid as dag
from dash import Dash, html, dcc, Input, Output, no_update, ctx
import json
from . import utils


def test_rs001_rowdata_sync(dash_duo):
    app = Dash(__name__)

    columnDefs = [
        {
            "headerName": "Make",
            "field": "make",
        },
        {
            "headerName": "Model",
            "field": "model",
        },
        {"headerName": "Price", "field": "price"},
    ]

    rowData = [
        {"make": "Toyota", "model": "Celica", "price": 35000},
        {"make": "Ford", "model": "Mondeo", "price": 32000},
        {"make": "Porsche", "model": "Boxster", "price": 72000},
    ]

    extraData = [
        {"make": "Subaru", "model": "Impreza", "price": 35000},
        {"make": "Mazda", "model": "CX-5", "price": 32000},
        {"make": "Honda", "model": "Pilot", "price": 72000},
    ]

    defaultColDef = {"editable": True}

    app.layout = html.Div(
        [
            dcc.Markdown(
                "In this grid, the __Make__ column has a popup below the cell,  the __Model__ has a popup above the cell, and the __Price__ has the default (in cell) editor."
            ),
            dag.AgGrid(
                columnDefs=columnDefs,
                rowData=rowData,
                defaultColDef=defaultColDef,
                columnSize="sizeToFit",
                id="grid",
                dashGridOptions={"editType": "fullRow"},
            ),
            html.Div(id="data"),
            html.Div(id="virtualData"),
            html.Button("edit mode", id="editMode", n_clicks=0),
            html.Button(id="reset", children="reset", n_clicks=0),
            html.Button("addRow", id="addRow", n_clicks=0),
            html.Button("addRow_notAsync", id="addRow_notAsync", n_clicks=0),
        ],
        style={"margin": 20},
    )

    @app.callback(Output("grid", "dashGridOptions"), Input("editMode", "n_clicks"))
    def toggleMode(n):
        if n:
            assert n > 0
            if n % 2 == 0:
                return {"editType": "fullRow"}
            else:
                return {"editType": None}

    @app.callback(
        Output("data", "children"),
        Input("grid", "cellValueChanged"),
        Input("grid", "rowData"),
    )
    def rowDataUpdate(c, d):
        if d:
            return json.dumps(d)

    @app.callback(
        Output("grid", "rowData"),
        Output("addRow", "n_clicks"),
        Output("addRow_notAsync", "n_clicks"),
        Input("reset", "n_clicks"),
    )
    def reset(n):
        if n > 0:
            return rowData, 0, 0
        return no_update, no_update, no_update

    @app.callback(
        Output("grid", "rowTransaction"),
        Input("addRow", "n_clicks"),
        Input("addRow_notAsync", "n_clicks"),
    )
    def addRows(n, n2):
        if n or n2:
            if ctx.triggered_id == "addRow":
                return {"add": [extraData[n - 1]]}
            if ctx.triggered_id == "addRow_notAsync":
                return {"add": [extraData[n2 - 1]], "async": False}

    @app.callback(
        Output("virtualData", "children"),
        Input("grid", "cellValueChanged"),
        Input("grid", "virtualRowData"),
    )
    def virtualDataUpdate(c, d):
        if d:
            return json.dumps(d)

    dash_duo.start_server(app)

    grid = utils.Grid(dash_duo, "grid")

    grid.wait_for_cell_text(0, 0, "Toyota")

    base = (
        '[{"make": "Toyota", "model": "Celica", "price": 35000}, '
        '{"make": "Ford", "model": "Mondeo", "price": 32000}, '
        '{"make": "Porsche", "model": "Boxster", "price": 72000}'
    )

    ### testing editing sync
    for x in range(2):
        grid.get_cell(0, 0).click()
        grid.get_cell(0, 0).send_keys("t")
        grid.get_cell(1, 1).click()
        dash_duo.wait_for_text_to_equal(
            "#data",
            '[{"make": "t", "model": "Celica", "price": 35000}, '
            '{"make": "Ford", "model": "Mondeo", "price": 32000}, '
            '{"make": "Porsche", "model": "Boxster", "price": 72000}]',
        )
        dash_duo.wait_for_text_to_equal(
            "#virtualData",
            '[{"make": "t", "model": "Celica", "price": 35000}, '
            '{"make": "Ford", "model": "Mondeo", "price": 32000}, '
            '{"make": "Porsche", "model": "Boxster", "price": 72000}]',
        )
        grid.get_cell(1, 1).send_keys("t")
        grid.get_cell(2, 2).click()
        dash_duo.wait_for_text_to_equal(
            "#data",
            '[{"make": "t", "model": "Celica", "price": 35000}, '
            '{"make": "Ford", "model": "t", "price": 32000}, '
            '{"make": "Porsche", "model": "Boxster", "price": 72000}]',
        )
        dash_duo.wait_for_text_to_equal(
            "#virtualData",
            '[{"make": "t", "model": "Celica", "price": 35000}, '
            '{"make": "Ford", "model": "t", "price": 32000}, '
            '{"make": "Porsche", "model": "Boxster", "price": 72000}]',
        )
        grid.get_cell(2, 2).send_keys("t")
        grid.get_cell(0, 0).click()
        dash_duo.wait_for_text_to_equal(
            "#data",
            '[{"make": "t", "model": "Celica", "price": 35000}, '
            '{"make": "Ford", "model": "t", "price": 32000}, '
            '{"make": "Porsche", "model": "Boxster", "price": null}]',
        )
        dash_duo.wait_for_text_to_equal(
            "#virtualData",
            '[{"make": "t", "model": "Celica", "price": 35000}, '
            '{"make": "Ford", "model": "t", "price": 32000}, '
            '{"make": "Porsche", "model": "Boxster", "price": null}]',
        )
        dash_duo.find_element("#reset").click()
        dash_duo.wait_for_text_to_equal("#data", base + "]")
        dash_duo.find_element("#editMode").click()

    for x in ["addRow", "addRow_notAsync"]:
        dash_duo.find_element(f"#{x}").click()
        dash_duo.wait_for_text_to_equal(
            "#data", base + ', {"make": "Subaru", "model": "Impreza", "price": 35000}]'
        )
        dash_duo.wait_for_text_to_equal(
            "#virtualData",
            base + ', {"make": "Subaru", "model": "Impreza", "price": 35000}]',
        )
        dash_duo.find_element(f"#{x}").click()
        dash_duo.wait_for_text_to_equal(
            "#data",
            base + ', {"make": "Subaru", "model": "Impreza", "price": 35000},'
            ' {"make": "Mazda", "model": "CX-5", "price": 32000}]',
        )
        dash_duo.wait_for_text_to_equal(
            "#virtualData",
            base + ', {"make": "Subaru", "model": "Impreza", "price": 35000},'
            ' {"make": "Mazda", "model": "CX-5", "price": 32000}]',
        )
        dash_duo.find_element(f"#{x}").click()
        dash_duo.wait_for_text_to_equal(
            "#data",
            base + ', {"make": "Subaru", "model": "Impreza", "price": 35000},'
            ' {"make": "Mazda", "model": "CX-5", "price": 32000}, '
            '{"make": "Honda", "model": "Pilot", "price": 72000}]',
        )
        dash_duo.wait_for_text_to_equal(
            "#virtualData",
            base + ', {"make": "Subaru", "model": "Impreza", "price": 35000},'
            ' {"make": "Mazda", "model": "CX-5", "price": 32000}, '
            '{"make": "Honda", "model": "Pilot", "price": 72000}]',
        )

        dash_duo.find_element("#reset").click()
        dash_duo.wait_for_text_to_equal("#data", base + "]")



import dash_ag_grid as dag
from dash import Dash, html, Input, Output
from . import utils


def test_rm001_row_menu(dash_duo):
    app = Dash(__name__)

    row_menu_example = html.Div(
        [
            dag.AgGrid(
                id="grid",
                columnSize="sizeToFit",
                getRowId="params.data.make",
                columnDefs=[
                    {"headerName": "Make", "field": "make", "sortable": True},
                    {"headerName": "Model", "field": "model"},
                    {"headerName": "Price", "field": "price"},
                    {"headerName": "Menu", "field": "menu", "cellRenderer": "rowMenu"},
                ],
                rowData=[
                    {
                        "make": "Toyota",
                        "model": "Celica",
                        "price": 35000,
                        "menu": [
                            {"label": "Option 1", "value": 1},
                            {"label": "Option 2", "value": 2},
                            {"label": "Option 3", "value": 3},
                        ],
                    },
                    {
                        "make": "Ford",
                        "model": "Mondeo",
                        "price": 32000,
                        "menu": [
                            {"label": "Option 4", "value": 4},
                            {"label": "Option 5", "value": 5},
                            {"label": "Option 6", "value": 6},
                        ],
                    },
                    {
                        "make": "Porsche",
                        "model": "Boxster",
                        "price": 72000,
                        "menu": [
                            {"label": "Option 7", "value": 7},
                            {"label": "Option 8", "value": 8},
                            {"label": "Option 9", "value": 9},
                        ],
                    },
                ],
            ),
            html.P(id="click-data"),
            html.Hr(),
        ]
    )

    app.layout = html.Div(
        row_menu_example,
        style={"flexWrap": "wrap"},
    )

    @app.callback(
        Output("click-data", "children"),
        Input("grid", "cellRendererData"),
    )
    def show_click_data(data):
        if data:
            assert "timestamp" in data
            return "You selected option {} from the colId {}, rowIndex {}, rowId {}.".format(
                data["value"],
                data["colId"],
                data["rowIndex"],
                data["rowId"],
            )
        return "No menu item selected."

    dash_duo.start_server(app)

    grid = utils.Grid(dash_duo, "grid")

    grid.wait_for_cell_text(0, 0, "Toyota")

    ### testing components
    grid.element_click_cell_button(0, 3)
    assert "opacity: 1" in dash_duo.find_element(".MuiMenu-paper").get_attribute(
        "style"
    )
    dash_duo.find_elements(".MuiMenu-paper .MuiMenu-list .MuiMenuItem-root")[1].click()
    dash_duo.wait_for_text_to_equal(
        "#click-data",
        "You selected option 2 from the colId menu, rowIndex 0, rowId Toyota.",
    )



import json
from setuptools import setup


with open("package.json") as f:
    package = json.load(f)

with open("README.md", encoding="utf-8") as f:
    long_description = f.read()


def read_req_file(req_type):
    with open(f"requires-{req_type}.txt", encoding="utf-8") as fp:
        requires = (line.strip() for line in fp)
        return [req for req in requires if req and not req.startswith("#")]


package_name = package["name"].replace(" ", "_").replace("-", "_")

setup(
    name=package_name,
    version=package["version"],
    author=package["author"],
    packages=[package_name],
    include_package_data=True,
    license=package["license"],
    description=package.get("description", package_name),
    long_description=long_description,
    long_description_content_type="text/markdown",
    install_requires=read_req_file("install"),
    extras_require={
        "dev": read_req_file("dev"),
        "docs": read_req_file("docs"),
    },
    python_requires=">=3.6",
    url="https://dash.plotly.com/dash-ag-grid",
    project_urls={
        "Documentation": "https://dash.plotly.com/dash-ag-grid",
        "Source": "https://github.com/plotly/dash-ag-grid",
        "Issue Tracker": "https://github.com/plotly/dash-ag-grid/issues",
    },
    classifiers=[
        "Framework :: Dash",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
    ],
)



import dash_ag_grid as dag
from dash import Dash, html, dcc
from . import utils
from dash.testing.wait import until
import pandas as pd
import time


def test_ce001_custom_cell_editor(dash_duo):
    rowData = [
        {"date": "2023-01-01"},
        {"date": "2023-02-11"},
        {"date": "2023-06-10"},
        {"date": "2023-11-04"},
        {"date": "2023-21-03"},
    ]

    # function to create a date object from  a date string "YYYY-MM-DD"
    date_obj = "d3.timeParse('%Y-%m-%d')(params.data.date)"

    columnDefs = [
        {"field": "date", "cellEditor": {"function": "DatePicker"}},
        {
            "headerName": "MM/DD/YYYY",
            "valueGetter": {"function": date_obj},
            "valueFormatter": {"function": f"d3.timeFormat('%m/%d/%Y')({date_obj})"},
        },
        {
            "headerName": "Mon DD, YYYY",
            "valueGetter": {"function": date_obj},
            "valueFormatter": {"function": f"d3.timeFormat('%b %d, %Y')({date_obj})"},
        },
        {
            "headerName": "day, Mon DD, YYYY",
            "valueGetter": {"function": date_obj},
            "valueFormatter": {
                "function": f"d3.timeFormat('%a %b %d, %Y')({date_obj})"
            },
        },
        {
            "headerName": "Month d, YYYY",
            "valueGetter": {"function": date_obj},
            "valueFormatter": {"function": f"d3.timeFormat('%B %e, %Y')({date_obj})"},
        },
        {
            "headerName": "MM-DD-YY",
            "valueGetter": {"function": date_obj},
            "valueFormatter": {"function": f"d3.timeFormat('%m-%d-%y')({date_obj})"},
        },
    ]

    defaultColDef = {
        "filter": "agDateColumnFilter",
        "filterParams": {
            "buttons": ["clear", "apply"],
        },
        "sortable": True,
        "editable": True,
    }

    app = Dash(
        __name__,
        external_scripts=[
            "https://cdnjs.cloudflare.com/ajax/libs/jquery/1.12.1/jquery.min.js",
            "https://cdnjs.cloudflare.com/ajax/libs/jqueryui/1.12.1/jquery-ui.min.js",
        ],
        external_stylesheets=[
            "https://cdnjs.cloudflare.com/ajax/libs/jqueryui/1.12.1/jquery-ui.min.css"
        ],
    )

    app.layout = html.Div(
        [
            dcc.Markdown("Date formatting example."),
            dag.AgGrid(
                columnDefs=columnDefs,
                rowData=rowData,
                defaultColDef=defaultColDef,
                id="grid",
            ),
        ],
        style={"margin": 20},
    )

    dash_duo.start_server(app)

    grid = utils.Grid(dash_duo, "grid")

    grid.wait_for_cell_text(0, 0, "2023-01-01")

    ### testing animations
    action = utils.ActionChains(dash_duo.driver)
    action.double_click(grid.get_cell(0, 0)).perform()
    until(
        lambda: "January"
        in dash_duo.find_element(".ui-datepicker-month").get_attribute("innerText"),
        timeout=3,
    )
    until(
        lambda: "2023"
        in dash_duo.find_element(".ui-datepicker-year").get_attribute("innerText"),
        timeout=3,
    )
    until(
        lambda: "1"
        in dash_duo.find_element(".ui-state-active").get_attribute("innerText"),
        timeout=3,
    )

    # set date to 2023-01-11
    dash_duo.find_element(
        "#ui-datepicker-div .ui-datepicker-calendar >"
        " tbody > tr:nth-child(2) > td:nth-child(4) > a"
    ).click()
    grid.get_cell(1, 0).click()
    grid.wait_for_cell_text(0, 0, "2023-01-11")



import dash_ag_grid as dag
from dash import Dash, html
from . import utils
from dash.testing.wait import until


def test_ds001_default_styles(dash_duo):
    app = Dash(__name__)

    columnDefs = [
        {"headerName": "Make", "field": "make"},
        {"headerName": "Model", "field": "model"},
        {
            "headerName": "Price",
            "field": "price",
            "cellStyle": {"defaultStyle": {"color": "green"}},
        },
    ]

    rowData = [
        {"make": "Toyota", "model": "Celica", "price": 35000},
        {"make": "Ford", "model": "Mondeo", "price": 32000},
        {"make": "Porsche", "model": "Boxster", "price": 72000},
    ]

    grid_with_default_cell_styles = html.Div(
        [
            html.H3(children="Grid with Default Cell Styles"),
            dag.AgGrid(
                columnDefs=columnDefs,
                rowData=rowData,
                columnSize="sizeToFit",
                defaultColDef=dict(
                    resizable=True,
                    cellStyle={"defaultStyle": {"color": "blue"}},
                ),
                id="grid",
            ),
            html.Hr(),
        ]
    )

    app.layout = grid_with_default_cell_styles

    dash_duo.start_server(app)

    grid = utils.Grid(dash_duo, "grid")

    grid.wait_for_cell_text(0, 0, "Toyota")

    ### testing styles
    until(
        lambda: "color: blue" in grid.get_cell(0, 0).get_attribute("style"), timeout=3
    )
    until(
        lambda: "color: green" in grid.get_cell(0, 2).get_attribute("style"), timeout=3
    )
